@@ -732,7 +732,7 @@ int _gnutls_proc_x509_server_certificate(GNUTLS_STATE state, opaque * data,
 
 	if ((ret =
 	     _gnutls_auth_info_set(state, GNUTLS_CRD_CERTIFICATE,
-				   sizeof(CERTIFICATE_AUTH_INFO_INT))) <
+				   sizeof(CERTIFICATE_AUTH_INFO_INT), 1)) <
 	    0) {
 		gnutls_assert();
 		return ret;
@@ -812,10 +812,6 @@ int _gnutls_proc_x509_server_certificate(GNUTLS_STATE state, opaque * data,
 	}
 
 
-	/* keep the PK algorithm */
-	state->gnutls_internals.peer_pk_algorithm =
-	    peer_certificate_list[0].subject_pk_algorithm;
-
 	if ((ret =
 	     _gnutls_copy_certificate_auth_info(info,
 						peer_certificate_list,
@@ -867,7 +863,7 @@ int _gnutls_proc_openpgp_server_certificate(GNUTLS_STATE state,
 
 	if ((ret =
 	     _gnutls_auth_info_set(state, GNUTLS_CRD_CERTIFICATE,
-				   sizeof(CERTIFICATE_AUTH_INFO_INT))) <
+				   sizeof(CERTIFICATE_AUTH_INFO_INT), 1)) <
 	    0) {
 		gnutls_assert();
 		return ret;
@@ -966,10 +962,6 @@ int _gnutls_proc_openpgp_server_certificate(GNUTLS_STATE state,
 	}
 	gnutls_free_datum( &akey);
 
-	/* keep the PK algorithm */
-	state->gnutls_internals.peer_pk_algorithm =
-	    peer_certificate_list[0].subject_pk_algorithm;
-
 	if ((ret =
 	     _gnutls_copy_certificate_auth_info(info,
 						peer_certificate_list,
@@ -1056,7 +1048,7 @@ int _gnutls_proc_cert_cert_req(GNUTLS_STATE state, opaque * data,
 
 	if ((ret =
 	     _gnutls_auth_info_set(state, GNUTLS_CRD_CERTIFICATE,
-				   sizeof(CERTIFICATE_AUTH_INFO_INT))) <
+				   sizeof(CERTIFICATE_AUTH_INFO_INT), 0)) <
 	    0) {
 		gnutls_assert();
 		return ret;
