@@ -30,6 +30,7 @@
 #include "gnutls_random.h"
 #include "gnutls_dh.h"
 #include "debug.h"
+#include <gnutls_str.h>
 
 /* this function parses tpasswd.conf file. Format is:
  * string(username):base64(v):base64(salt):int(index)
@@ -110,15 +111,21 @@ int indx;
 
 	if (_gnutls_mpi_scan(&entry->v, verifier, &verifier_size) || entry->v == NULL) {
 		gnutls_assert();
-		gnutls_free(entry->salt);
+		gnutls_free( entry->salt);
 		return GNUTLS_E_MPI_SCAN_FAILED;
 	}
 
+	gnutls_free( verifier);
 
 	/* now go for username */
 	*p='\0';
 
 	entry->username = gnutls_strdup(str);
+	if (entry->username==NULL) {
+		gnutls_free( entry->salt);
+		gnutls_assert();
+		return GNUTLS_E_MEMORY_ERROR;
+	}
 
 	return indx;
 }
@@ -194,15 +201,15 @@ int tmp_size;
 
 /* this function opens the tpasswd.conf file
  */
-static int pwd_read_conf( const GNUTLS_SRP_SERVER_CREDENTIALS cred, GNUTLS_SRP_PWD_ENTRY* entry, int index) {
+static int pwd_read_conf( const char* pconf_file, GNUTLS_SRP_PWD_ENTRY* entry, int index) {
 	FILE * fd;
-	char line[5*1024];
+	char line[2*1024];
 	int i;
 	char indexstr[10];
 
 	sprintf( indexstr, "%d", index); /* Flawfinder: ignore */
 
-	fd = fopen( cred->password_conf_file, "r");
+	fd = fopen( pconf_file, "r");
 	if (fd==NULL) {
 		gnutls_assert();
 		gnutls_free(entry);
@@ -228,14 +235,15 @@ static int pwd_read_conf( const GNUTLS_SRP_SERVER_CREDENTIALS cred, GNUTLS_SRP_P
 }
 
 
-GNUTLS_SRP_PWD_ENTRY *_gnutls_srp_pwd_read_entry( GNUTLS_KEY key, char* username, int *err) {
+GNUTLS_SRP_PWD_ENTRY *_gnutls_srp_pwd_read_entry( GNUTLS_STATE state, char* username, int *err) {
 	const GNUTLS_SRP_SERVER_CREDENTIALS cred;
 	FILE * fd;
-	char line[5*1024];
+	char line[2*1024];
 	int i, len;
 	GNUTLS_SRP_PWD_ENTRY * entry = gnutls_malloc(sizeof(GNUTLS_SRP_PWD_ENTRY));
 	int index;
-
+	int pwd_index = 0;
+	
 	if (entry==NULL) {
 		gnutls_assert();
 		*err = 1;
@@ -244,7 +252,7 @@ GNUTLS_SRP_PWD_ENTRY *_gnutls_srp_pwd_read_entry( GNUTLS_KEY key, char* username
 
 	*err = 0; /* normal exit */
 	
-	cred = _gnutls_get_cred( key, GNUTLS_SRP, NULL);
+	cred = _gnutls_get_cred( state->gnutls_key, GNUTLS_CRD_SRP, NULL);
 	if (cred==NULL) {
 		*err = 1;
 		gnutls_assert();
@@ -252,7 +260,24 @@ GNUTLS_SRP_PWD_ENTRY *_gnutls_srp_pwd_read_entry( GNUTLS_KEY key, char* username
 		return NULL;
 	}
 
-	fd = fopen( cred->password_file, "r");
+	if (cred->password_files<=0) {
+		gnutls_assert();
+		return NULL;
+	}
+	
+	/* use the callback to select a password file */
+	if (state->gnutls_internals.server_srp_callback!=NULL) {
+		pwd_index = state->gnutls_internals.server_srp_callback(
+			state, cred->password_file, cred->password_conf_file,
+				cred->password_files);
+
+		if (pwd_index < 0) {
+			gnutls_assert();
+			return NULL;
+		}
+	}
+
+	fd = fopen( cred->password_file[pwd_index], "r");
 	if (fd==NULL) {
 		*err = 1; /* failed due to critical error */
 		gnutls_assert();
@@ -269,7 +294,7 @@ GNUTLS_SRP_PWD_ENTRY *_gnutls_srp_pwd_read_entry( GNUTLS_KEY key, char* username
 	    len = strlen(username);
 	    if (strncmp( username, line, (i>len)?i:len) == 0) {
 			if ((index = pwd_put_values( entry, line, strlen(line))) >= 0)
-				if (pwd_read_conf( cred, entry, index)==0) {
+				if (pwd_read_conf( cred->password_conf_file[pwd_index], entry, index)==0) {
 					return entry;
 				} else {
 					gnutls_free(entry);
@@ -294,7 +319,7 @@ GNUTLS_SRP_PWD_ENTRY* _gnutls_randomize_pwd_entry() {
 		return NULL;
 	}
 	
-	pwd_entry->g = gnutls_get_dh_params( &pwd_entry->n, 1024);
+	pwd_entry->g = _gnutls_get_rnd_srp_params( &pwd_entry->n, 1024);
 	if (pwd_entry->g==NULL || pwd_entry->n==NULL) {
 		gnutls_assert();
 		_gnutls_srp_clear_pwd_entry( pwd_entry);
@@ -307,7 +332,7 @@ GNUTLS_SRP_PWD_ENTRY* _gnutls_randomize_pwd_entry() {
 		_gnutls_srp_clear_pwd_entry( pwd_entry);
 		return NULL;
 	}
-	strcpy( pwd_entry->username, RNDUSER); /* Flawfinder: ignore */
+	_gnutls_str_cpy( pwd_entry->username, MAX_SRP_USERNAME, RNDUSER); /* Flawfinder: ignore */
 	
 	pwd_entry->v = _gnutls_mpi_new(160);
 	if (pwd_entry->v==NULL) {
