@@ -25,7 +25,7 @@
 extern "C" {
 #endif
 
-#define LIBGNUTLS_VERSION "0.3.90"
+#define LIBGNUTLS_VERSION "0.3.91"
 
 #include <sys/types.h>
 #include <time.h>
@@ -82,7 +82,6 @@ typedef enum GNUTLS_CertificateType { GNUTLS_CRT_X509=1, GNUTLS_CRT_OPENPGP
  * define in gnutls_int.h, and recompile.
  */
 #define GNUTLS_TRANSPORT_PTR int
-#define GNUTLS_SOCKET_PTR GNUTLS_TRANSPORT_PTR
 
 typedef const int* GNUTLS_LIST;
 
@@ -133,7 +132,7 @@ int gnutls_error_is_fatal( int error);
 void gnutls_perror( int error);
 const char* gnutls_strerror( int error);
 
-ssize_t gnutls_record_send( GNUTLS_STATE state, void *data, size_t sizeofdata);
+ssize_t gnutls_record_send( GNUTLS_STATE state, const void *data, size_t sizeofdata);
 ssize_t gnutls_record_recv( GNUTLS_STATE state, void *data, size_t sizeofdata);
 #define gnutls_read gnutls_record_recv
 #define gnutls_write gnutls_record_send
@@ -166,6 +165,7 @@ void gnutls_db_set_cache_expiration( GNUTLS_STATE state, int seconds);
 
 int  gnutls_db_set_name( GNUTLS_STATE state, const char* filename);	
 int  gnutls_db_clean( GNUTLS_STATE state);
+void  gnutls_db_remove_session( GNUTLS_STATE state);
 void gnutls_db_set_retrieve_func( GNUTLS_STATE, GNUTLS_DB_RETR_FUNC);
 void gnutls_db_set_remove_func( GNUTLS_STATE, GNUTLS_DB_REMOVE_FUNC);
 void gnutls_db_set_store_func( GNUTLS_STATE, GNUTLS_DB_STORE_FUNC);
