@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2000,2001 Nikos Mavroyanopoulos
+ *      Copyright (C) 2000,2001,2002 Nikos Mavroyanopoulos
  *
  * This file is part of GNUTLS.
  *
@@ -22,6 +22,47 @@
 #include <gnutls_errors.h>
 #include <gnutls_alert.h>
 #include <gnutls_record.h>
+#include <debug.h>
+
+typedef struct {
+	AlertDescription alert;
+	char *desc;
+} gnutls_alert_entry;
+
+static const gnutls_alert_entry sup_alerts[] = {
+	{ GNUTLS_A_CLOSE_NOTIFY, "Close notify" },
+	{ GNUTLS_A_UNEXPECTED_MESSAGE,	 "Unexpected message" },
+	{ GNUTLS_A_BAD_RECORD_MAC,	 "Bad record MAC" },
+	{ GNUTLS_A_DECRYPTION_FAILED,	 "Decryption failed" },
+	{ GNUTLS_A_RECORD_OVERFLOW,	 "Record overflow" }, 
+	{ GNUTLS_A_DECOMPRESSION_FAILURE, "Decompression failed" },
+	{ GNUTLS_A_HANDSHAKE_FAILURE,	 "Handshake failed" },
+	{ GNUTLS_A_BAD_CERTIFICATE,	 "Certificate is bad" },
+	{ GNUTLS_A_UNSUPPORTED_CERTIFICATE,	 "Certificate is not supported" },
+	{ GNUTLS_A_CERTIFICATE_REVOKED,	 "Certificate was revoked" },
+	{ GNUTLS_A_CERTIFICATE_EXPIRED,	 "Certificate is expired" },
+	{ GNUTLS_A_CERTIFICATE_UNKNOWN,	 "Unknown certificate" },
+	{ GNUTLS_A_ILLEGAL_PARAMETER,	 "Illegal parameter" },
+	{ GNUTLS_A_UNKNOWN_CA,	 	 "CA is unknown" },
+	{ GNUTLS_A_ACCESS_DENIED,	 "Access was denied" },
+	{ GNUTLS_A_DECODE_ERROR,	 "Decode error" },
+	{ GNUTLS_A_DECRYPT_ERROR,	 "Decrypt error" },
+	{ GNUTLS_A_EXPORT_RESTRICTION,	 "Export restriction" },
+	{ GNUTLS_A_PROTOCOL_VERSION,	 "Error in protocol version" },
+	{ GNUTLS_A_INSUFFICIENT_SECURITY,"Insufficient security" },
+	{ GNUTLS_A_USER_CANCELED,	 "User canceled" },
+	{ GNUTLS_A_NO_RENEGOTIATION,	 "No renegotiation is allowed" },
+	{0, NULL}
+};
+
+#define GNUTLS_ALERT_LOOP(b) \
+        const gnutls_alert_entry *p; \
+                for(p = sup_alerts; p->desc != NULL; p++) { b ; }
+
+#define GNUTLS_ALERT_ID_LOOP(a) \
+                        GNUTLS_ALERT_LOOP( if(p->alert == alert) { a; break; })
+
+
 
 /**
   * gnutls_alert_send - This function sends an alert message to the peer
@@ -36,7 +77,7 @@
   * Returns 0 on success.
   *
   **/
-int gnutls_alert_send( GNUTLS_STATE state, AlertLevel level, AlertDescription desc)
+int gnutls_alert_send( GNUTLS_STATE state, GNUTLS_AlertLevel level, GNUTLS_AlertDescription desc)
 {
 	uint8 data[2];
 	int ret;
@@ -44,9 +85,7 @@ int gnutls_alert_send( GNUTLS_STATE state, AlertLevel level, AlertDescription de
 	data[0] = (uint8) level;
 	data[1] = (uint8) desc;
 
-#ifdef RECORD_DEBUG
-	_gnutls_log( "Record: Sending Alert[%d|%d] - %s\n", data[0], data[1], _gnutls_alert2str((int)data[1]));
-#endif
+	_gnutls_record_log( "REC: Sending Alert[%d|%d] - %s\n", data[0], data[1], _gnutls_alert_get_name((int)data[1]));
 
 	if ( (ret = gnutls_send_int( state, GNUTLS_ALERT, -1, data, 2)) >= 0)
 		return 0;
@@ -77,11 +116,11 @@ int gnutls_alert_send( GNUTLS_STATE state, AlertLevel level, AlertDescription de
 int gnutls_alert_send_appropriate( GNUTLS_STATE state, int err) {
 int ret = GNUTLS_E_UNIMPLEMENTED_FEATURE;
 	switch (err) { /* send appropriate alert */
-		case GNUTLS_E_MAC_FAILED:
-			ret = gnutls_alert_send( state, GNUTLS_AL_FATAL, GNUTLS_A_BAD_RECORD_MAC);
-			break;
 		case GNUTLS_E_DECRYPTION_FAILED:
-			ret = gnutls_alert_send( state, GNUTLS_AL_FATAL, GNUTLS_A_DECRYPTION_FAILED);
+			/* GNUTLS_A_DECRYPTION_FAILED is not sent, because
+			 * it is not defined in SSL3.
+			 */
+			ret = gnutls_alert_send( state, GNUTLS_AL_FATAL, GNUTLS_A_BAD_RECORD_MAC);
 			break;
 		case GNUTLS_E_DECOMPRESSION_FAILED:
 			ret = gnutls_alert_send( state, GNUTLS_AL_FATAL, GNUTLS_A_DECOMPRESSION_FAILURE);
@@ -105,6 +144,9 @@ int ret = GNUTLS_E_UNIMPLEMENTED_FEATURE;
 		case GNUTLS_E_UNSUPPORTED_VERSION_PACKET:
                         ret = gnutls_alert_send( state, GNUTLS_AL_WARNING, GNUTLS_A_PROTOCOL_VERSION);
 			break;
+		case GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE:
+                        ret = gnutls_alert_send( state, GNUTLS_AL_WARNING, GNUTLS_A_UNSUPPORTED_CERTIFICATE);
+			break;
 		case GNUTLS_E_UNEXPECTED_PACKET_LENGTH:
 			ret = gnutls_alert_send( state, GNUTLS_AL_FATAL, GNUTLS_A_RECORD_OVERFLOW);
 			break;
@@ -113,7 +155,7 @@ int ret = GNUTLS_E_UNIMPLEMENTED_FEATURE;
 }
 
 /**
-  * gnutls_alert_get_last - Returns the last alert number received.
+  * gnutls_alert_get - Returns the last alert number received.
   * @state: is a &GNUTLS_STATE structure.
   *
   * Returns the last alert number received. This function
@@ -122,6 +164,22 @@ int ret = GNUTLS_E_UNIMPLEMENTED_FEATURE;
   * The peer may send alerts if he thinks some things were not 
   * right. Check gnutls.h for the available alert descriptions.
   **/
-AlertDescription gnutls_alert_get_last( GNUTLS_STATE state) {
+GNUTLS_AlertDescription gnutls_alert_get( GNUTLS_STATE state) {
 	return state->gnutls_internals.last_alert;
 }
+
+/**
+  * gnutls_alert_get_name - Returns a string describing the alert number given
+  * @alert: is an alert number &GNUTLS_STATE structure.
+  *
+  * Returns a string that describes the given alert number.
+  * See. gnutls_alert_get().
+  *
+  **/
+const char* gnutls_alert_get_name( int alert) {
+char* ret = NULL;
+
+	GNUTLS_ALERT_ID_LOOP( ret = p->desc);
+
+	return ret;
+}
