@@ -233,7 +233,12 @@ void _gnutls_free_auth_info( GNUTLS_STATE state) {
 
 }
 
-int _gnutls_auth_info_set( GNUTLS_STATE state, CredType type, int size) {
+/* This function will set the auth info structure in the gnutls_key
+ * structure.
+ * If allow change is !=0 then this will allow changing the auth
+ * info structure to a different type.
+ */
+int _gnutls_auth_info_set( GNUTLS_STATE state, CredType type, int size, int allow_change) {
 	if ( state->gnutls_key->auth_info == NULL) {
 		state->gnutls_key->auth_info = gnutls_calloc( 1, size);
 		if (state->gnutls_key->auth_info == NULL) {
@@ -242,17 +247,37 @@ int _gnutls_auth_info_set( GNUTLS_STATE state, CredType type, int size) {
 		}
 		state->gnutls_key->auth_info_type = type;
 		state->gnutls_key->auth_info_size = size;
-	} else
-		/* If the credentials for the current authentication scheme,
-		 * are not the one we want to set, then it's an error.
-		 * This may happen if a rehandshake is performed an the
-		 * ciphersuite which is negotiated has different authentication
-		 * schema.
-		 */
-		if ( gnutls_auth_get_type( state) != state->gnutls_key->auth_info_type) {
-			gnutls_assert();
-			return GNUTLS_E_INVALID_REQUEST;
+	} else {
+		if (allow_change==0) {
+			/* If the credentials for the current authentication scheme,
+			 * are not the one we want to set, then it's an error.
+			 * This may happen if a rehandshake is performed an the
+			 * ciphersuite which is negotiated has different authentication
+			 * schema.
+			 */
+			if ( gnutls_auth_get_type( state) != state->gnutls_key->auth_info_type) {
+				gnutls_assert();
+				return GNUTLS_E_INVALID_REQUEST;
+			}
+		} else {
+			/* The new behaviour: Here we reallocate the auth info structure
+			 * in order to be able to negotiate different authentication
+			 * types. Ie. perform an auth_anon and then authenticate again using a
+			 * certificate (in order to prevent revealing the certificate's contents,
+			 * to passive eavesdropers.
+			 */
+			if ( gnutls_auth_get_type( state) != state->gnutls_key->auth_info_type) {
+				state->gnutls_key->auth_info = gnutls_realloc_fast( 
+					state->gnutls_key->auth_info, size);
+				if (state->gnutls_key->auth_info == NULL) {
+					gnutls_assert();
+					return GNUTLS_E_MEMORY_ERROR;
+				}
+				memset( state->gnutls_key->auth_info, 0, size);
+				state->gnutls_key->auth_info_type = type;
+				state->gnutls_key->auth_info_size = size;
+			}
 		}
-
+	}
 	return 0;
 }
