@@ -94,7 +94,7 @@ static gnutls_error_entry error_algorithms[] = {
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_AGAIN, 0),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_INTERRUPTED, 0),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_REHANDSHAKE, 0),
-	GNUTLS_ERROR_ENTRY( GNUTLS_E_GOT_APPLICATION_DATA, 0),
+	GNUTLS_ERROR_ENTRY( GNUTLS_E_GOT_APPLICATION_DATA, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_DB_ERROR, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE, 1),
 	GNUTLS_ERROR_ENTRY( GNUTLS_E_INVALID_PARAMETERS, 1),
@@ -147,7 +147,7 @@ void gnutls_perror(int error)
 	GNUTLS_ERROR_ALG_LOOP(ret =
 			      gnutls_strdup(p->name + sizeof("GNUTLS_E_") - 1));
 
-	_gnutls_log( "GNUTLS ERROR: %s\n", ret);
+	fprintf(stderr,  "GNUTLS ERROR: %s\n", ret);
 	
 	gnutls_free( ret);
 }
