@@ -23,6 +23,7 @@
 #include "gnutls_errors.h"
 #include "ext_srp.h"
 #include "ext_max_record.h"
+#include <ext_cert_type.h>
 #include "gnutls_num.h"
 
 /* Key Exchange Section */
@@ -31,15 +32,15 @@
 
 typedef struct {
 	char *name;
-	int type;
+	uint16 type;
 	int (*gnutls_ext_func_recv)( GNUTLS_STATE, const opaque*, int); /* recv data */
 	int (*gnutls_ext_func_send)( GNUTLS_STATE, opaque**); /* send data */
 } gnutls_extension_entry;
 
-#define MAX_EXT 20 /* maximum supported extension */
 static gnutls_extension_entry extensions[] = {
 	GNUTLS_EXTENSION_ENTRY( GNUTLS_EXTENSION_SRP, _gnutls_srp_recv_params, _gnutls_srp_send_params),
 	GNUTLS_EXTENSION_ENTRY( GNUTLS_EXTENSION_MAX_RECORD_SIZE, _gnutls_max_record_recv_params, _gnutls_max_record_send_params),
+	GNUTLS_EXTENSION_ENTRY( GNUTLS_EXTENSION_CERT_TYPE, _gnutls_cert_type_recv_params, _gnutls_cert_type_send_params),
 	{0}
 };
 
@@ -53,14 +54,14 @@ static gnutls_extension_entry extensions[] = {
 
 /* EXTENSION functions */
 
-void* _gnutls_ext_func_recv(int type)
+void* _gnutls_ext_func_recv(uint16 type)
 {
 	void* ret=NULL;
 	GNUTLS_EXTENSION_LOOP(ret = p->gnutls_ext_func_recv);
 	return ret;
 
 }
-void* _gnutls_ext_func_send(int type)
+void* _gnutls_ext_func_send(uint16 type)
 {
 	void* ret=NULL;
 	GNUTLS_EXTENSION_LOOP(ret = p->gnutls_ext_func_send);
@@ -68,7 +69,7 @@ void* _gnutls_ext_func_send(int type)
 
 }
 
-const char *_gnutls_extension_get_name(int type)
+const char *_gnutls_extension_get_name(uint16 type)
 {
 	char *ret = NULL;
 
@@ -78,14 +79,39 @@ const char *_gnutls_extension_get_name(int type)
 	return ret;
 }
 
+/* Checks if the extension we just received is one of the 
+ * requested ones. Otherwise it's a fatal error.
+ */
+static int _gnutls_extension_list_check( GNUTLS_STATE state, uint8 type) {
+int i;
+	if (state->security_parameters.entity==GNUTLS_CLIENT) {
+		for(i=0;i<state->gnutls_internals.extensions_sent_size;i++) {
+			if (type==state->gnutls_internals.extensions_sent[i])
+				return 0; /* ok found */
+		}
+		return GNUTLS_E_RECEIVED_ILLEGAL_EXTENSION;
+	}
+
+	return 0;
+}
+
 int _gnutls_parse_extensions( GNUTLS_STATE state, const opaque* data, int data_size) {
 int next, ret;
 int pos=0;
-uint8 type;
+uint16 type;
 const opaque* sdata;
 int (*ext_func_recv)( GNUTLS_STATE, const opaque*, int);
 uint16 size;
 
+#ifdef DEBUG
+int i;
+
+	if (state->security_parameters.entity==GNUTLS_CLIENT)
+		for (i=0;i<state->gnutls_internals.extensions_sent_size;i++) {
+			_gnutls_log("EXT: expecting extension %d\n", state->gnutls_internals.extensions_sent[i]);
+		}
+#endif
+
 	DECR_LENGTH_RET( data_size, 2, 0);
 	next = READuint16( data);
 	pos+=2;
@@ -94,7 +120,13 @@ uint16 size;
 	
 	do {
 		DECR_LENGTH_RET( next, 1, 0);
-		type = data[pos++];
+		type = READuint16( &data[pos]);
+		pos+=2;
+		
+		if ( (ret=_gnutls_extension_list_check( state, type)) < 0) {
+			gnutls_assert();
+			return ret;
+		}
 
 		DECR_LENGTH_RET( next, 2, 0);
 		size = READuint16(&data[pos]);
@@ -117,6 +149,28 @@ uint16 size;
 
 }
 
+/* Adds the extension we want to send in the extensions list.
+ * This list is used to check whether the (later) received
+ * extensions are the ones we requested.
+ */
+static void _gnutls_extension_list_add( GNUTLS_STATE state, uint8 type) {
+
+	if (state->security_parameters.entity==GNUTLS_CLIENT) {
+		if (state->gnutls_internals.extensions_sent_size <
+			sizeof(state->gnutls_internals.extensions_sent)) {
+	
+			state->gnutls_internals.extensions_sent[state->gnutls_internals.extensions_sent_size] = type;
+			state->gnutls_internals.extensions_sent_size++;
+		} else {
+#ifdef DEBUG
+			_gnutls_log("EXT: Increase MAX_EXT_TYPES\n");
+#endif
+		}
+	}
+
+	return;
+}
+
 int _gnutls_gen_extensions( GNUTLS_STATE state, opaque** data) {
 int next, size;
 uint16 pos=0;
@@ -127,22 +181,40 @@ int (*ext_func_send)( GNUTLS_STATE, opaque**);
 	(*data) = gnutls_malloc(2); /* allocate size for size */
 	pos+=2;
 	
-	next = MAX_EXT; /* maximum supported extensions */
+	if ((*data)==NULL) {
+		gnutls_assert();
+		return GNUTLS_E_MEMORY_ERROR;
+	}
+	
+	next = MAX_EXT_TYPES; /* maximum supported extensions */
 	do {
 		next--;
 		ext_func_send = _gnutls_ext_func_send(next);
 		if (ext_func_send == NULL) continue;
 		size = ext_func_send( state, &sdata);
-		if (size > 0) {
-			(*data) = gnutls_realloc( (*data), pos+size+3);
-			(*data)[pos++] = (uint8) next; /* set type */
 
+		if (size > 0) {
+			(*data) = gnutls_realloc( (*data), pos+size+4);
+			if ((*data)==NULL) {
+				gnutls_assert();
+				return GNUTLS_E_MEMORY_ERROR;
+			}
+
+			/* write extension type */
+			WRITEuint16( next, &(*data)[pos]);
+			pos+=2;
+			
+			/* write size */
 			WRITEuint16( size, &(*data)[pos]);
 			pos+=2;
 			
 			memcpy( &(*data)[pos], sdata, size);
 			pos+=size;
 			gnutls_free(sdata);
+			
+			/* add this extension to the extension list
+			 */
+			_gnutls_extension_list_add( state, next);
 		}
 		
 	} while(next >= 0);
