@@ -19,6 +19,8 @@
  */
 
 #include <gnutls_int.h>
+#include <x509_asn1.h>
+#include <gnutls_errors.h>
 
 /* Functions that refer to the libgcrypt library.
  */
@@ -42,3 +44,30 @@ int _gnutls_mpi_print( opaque *buffer, size_t *nbytes, const GCRY_MPI a ) {
 	return gcry_mpi_print( GCRYMPI_FMT_USG, buffer, nbytes, a);
 }
 
+/* Always has the first bit zero */
+int _gnutls_mpi_print_lz( opaque *buffer, size_t *nbytes, const GCRY_MPI a ) {
+	return gcry_mpi_print( GCRYMPI_FMT_STD, buffer, nbytes, a);
+}
+
+
+/* this function reads an integer
+ * from asn1 structs. Combines the read and mpi_scan
+ * steps.
+ */
+int _gnutls_x509_read_int( node_asn* node, char* value, char* tmpstr, int tmpstr_size, MPI* ret_mpi) {
+int len, result;
+
+	len = tmpstr_size - 1;
+	result = asn1_read_value(node, value, tmpstr, &len);
+	if (result != ASN_OK) {
+		gnutls_assert();
+		return GNUTLS_E_ASN1_PARSING_ERROR;
+	}
+
+	if (_gnutls_mpi_scan( ret_mpi, tmpstr, &len) != 0) {
+		gnutls_assert();
+		return GNUTLS_E_MPI_SCAN_FAILED;
+	}
+
+	return 0;
+}
