@@ -335,13 +335,15 @@ int _gnutls_read_client_hello(GNUTLS_STATE state, opaque * data,
 	}
 	
 	/* Parse the extensions (if any)
-	 */	
-	ret = _gnutls_parse_extensions(state, &data[pos], len);	/* len is the rest of the parsed length */
-	if (ret < 0) {
-		gnutls_assert();
-		return ret;
+	 */
+	if (ver >= GNUTLS_TLS1) {
+		ret = _gnutls_parse_extensions(state, &data[pos], len);	/* len is the rest of the parsed length */
+		if (ret < 0) {
+			gnutls_assert();
+			return ret;
+		}
 	}
-
+	
 	/* select an appropriate cipher suite
 	 */
 	ret = _gnutls_server_select_suite(state, suite_ptr, sizeOfSuites);
@@ -1286,10 +1288,12 @@ static int _gnutls_read_server_hello(GNUTLS_STATE state, char *data,
 
 	/* Parse extensions.
 	 */
-	ret = _gnutls_parse_extensions(state, &data[pos], len);	/* len is the rest of the parsed length */
-	if (ret < 0) {
-		gnutls_assert();
-		return ret;
+	if (version >= GNUTLS_TLS1) {
+		ret = _gnutls_parse_extensions(state, &data[pos], len);	/* len is the rest of the parsed length */
+		if (ret < 0) {
+			gnutls_assert();
+			return ret;
+		}
 	}
 	return ret;
 }
@@ -1469,7 +1473,10 @@ static int _gnutls_send_client_hello(GNUTLS_STATE state, int again)
 		 * handshake packet and ignore the one in the packet's record 
 		 * header.
 		 */
-		_gnutls_set_current_version(state, hver);
+		if (state->gnutls_internals.default_record_version==0)
+			_gnutls_set_current_version(state, hver);
+		else _gnutls_set_current_version(state,
+			state->gnutls_internals.default_record_version);
 
 		/* In order to know when this session was initiated.
 		 */
@@ -1544,20 +1551,21 @@ static int _gnutls_send_client_hello(GNUTLS_STATE state, int again)
 
 		/* Generate and copy TLS extensions.
 		 */
-		extdatalen = _gnutls_gen_extensions(state, &extdata);
-		if (extdatalen > 0) {
-			datalen += extdatalen;
-			data = gnutls_realloc(data, datalen);
-			if (data == NULL) {
-				gnutls_assert();
+		if (hver >= GNUTLS_TLS1) {
+			extdatalen = _gnutls_gen_extensions(state, &extdata);
+			if (extdatalen > 0) {
+				datalen += extdatalen;
+				data = gnutls_realloc(data, datalen);
+				if (data == NULL) {
+					gnutls_assert();
+					gnutls_free(extdata);
+					return GNUTLS_E_MEMORY_ERROR;
+				}
+
+				memcpy(&data[pos], extdata, extdatalen);
 				gnutls_free(extdata);
-				return GNUTLS_E_MEMORY_ERROR;
 			}
-
-			memcpy(&data[pos], extdata, extdatalen);
-			gnutls_free(extdata);
 		}
-
 	}
 
 	ret =
@@ -1850,6 +1858,7 @@ int gnutls_handshake(GNUTLS_STATE state)
 	STATE = STATE0;
 
 	_gnutls_handshake_io_buffer_clear(state);
+	_gnutls_handshake_internal_state_clear(state);
 
 	return 0;
 }
@@ -2286,8 +2295,11 @@ int _gnutls_recv_hello_request(GNUTLS_STATE state, void *data,
 }
 
 /* This function will remove algorithms that are not supported by
- * the requested authentication method. We only remove algorithm if
+ * the requested authentication method. We remove algorithm if
  * we have a certificate with keyUsage bits set.
+ *
+ * This does a more high level check than  gnutls_supported_ciphersuites(),
+ * by checking certificates etc.
  */
 int _gnutls_remove_unwanted_ciphersuites(GNUTLS_STATE state,
 					 GNUTLS_CipherSuite **
