@@ -483,9 +483,6 @@ typedef struct {
 	 * supports it.
 	 */
 	int				send_cert_req;
-	int				peer_pk_algorithm;
-	/* holds the username got in the srp tls extension
-	 */
 
 	/* this is a callback function to call if no appropriate
 	 * client certificates were found.
@@ -542,6 +539,13 @@ typedef struct {
 	 * if the fingerprint is to be sent.
 	 */
 	int			pgp_fingerprint;
+
+	/* This holds the default version that our first
+	 * record packet will have. */
+	GNUTLS_Version		default_record_version;
+
+	/* If you add anything here, check _gnutls_handshake_internal_state_clear().
+	 */
 } GNUTLS_INTERNALS;
 
 struct GNUTLS_STATE_INT {
