@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2000,2001 Nikos Mavroyanopoulos
+ *      Copyright (C) 2000,2001,2002 Nikos Mavroyanopoulos
  *
  * This file is part of GNUTLS.
  *
@@ -36,6 +36,7 @@
 #include "gnutls_datum.h"
 #include "ext_max_record.h"
 #include <gnutls_alert.h>
+#include <gnutls_dh.h>
 
 /**
   * gnutls_protocol_get_version - Returns the version of the currently used protocol
@@ -45,9 +46,7 @@
   *
   **/
 GNUTLS_Version gnutls_protocol_get_version(GNUTLS_STATE state) {
-GNUTLS_Version ver;
-	ver = state->security_parameters.version;
-	return ver;
+	return state->security_parameters.version;
 }
 
 void _gnutls_set_current_version(GNUTLS_STATE state, GNUTLS_Version version) {
@@ -81,7 +80,7 @@ void gnutls_transport_set_lowat(GNUTLS_STATE state, int num) {
   * handle.
   *
   **/
-void gnutls_transport_set_ptr(GNUTLS_STATE state, GNUTLS_SOCKET_PTR ptr) {
+void gnutls_transport_set_ptr(GNUTLS_STATE state, GNUTLS_TRANSPORT_PTR ptr) {
 	state->gnutls_internals.transport_ptr = ptr;
 }
 
@@ -93,158 +92,10 @@ void gnutls_transport_set_ptr(GNUTLS_STATE state, GNUTLS_SOCKET_PTR ptr) {
   * PULL). This must have been set using gnutls_transport_set_ptr().
   *
   **/
-GNUTLS_SOCKET_PTR gnutls_transport_get_ptr(GNUTLS_STATE state) {
+GNUTLS_TRANSPORT_PTR gnutls_transport_get_ptr(GNUTLS_STATE state) {
 	return state->gnutls_internals.transport_ptr;
 }
 
-#define _gnutls_free(x) if(x!=NULL) gnutls_free(x)
-/**
-  * gnutls_init - This function initializes the state to null (null encryption etc...).
-  * @con_end: is used to indicate if this state is to be used for server or 
-  * client. Can be one of GNUTLS_CLIENT and GNUTLS_SERVER. 
-  * @state: is a pointer to a &GNUTLS_STATE structure.
-  *
-  * This function initializes the current state to null. Every state
-  * must be initialized before use, so internal structures can be allocated.
-  * This function allocates structures which can only be free'd
-  * by calling gnutls_deinit(). Returns zero on success.
-  **/
-int gnutls_init(GNUTLS_STATE * state, ConnectionEnd con_end)
-{
-int default_protocol_list[] = { GNUTLS_TLS1, 0 };
-
-	*state = gnutls_calloc(1, sizeof(struct GNUTLS_STATE_INT));
-	if (*state==NULL) return GNUTLS_E_MEMORY_ERROR;
-	
-	(*state)->security_parameters.entity = con_end;
-
-/* Set the defaults for initial handshake */
-	(*state)->security_parameters.read_bulk_cipher_algorithm = 
-	(*state)->security_parameters.write_bulk_cipher_algorithm = GNUTLS_CIPHER_NULL;
-
-	(*state)->security_parameters.read_mac_algorithm = 
-	(*state)->security_parameters.write_mac_algorithm = GNUTLS_MAC_NULL;
-
-	(*state)->security_parameters.read_compression_algorithm = GNUTLS_COMP_NULL;
-	(*state)->security_parameters.write_compression_algorithm = GNUTLS_COMP_NULL;
-
-	(*state)->gnutls_internals.resumable = RESUME_TRUE;
-
-	gnutls_protocol_set_priority( *state, default_protocol_list); /* default */
-	
-	(*state)->gnutls_key = gnutls_calloc(1, sizeof(struct GNUTLS_KEY_INT));
-	if ( (*state)->gnutls_key == NULL) {
-		gnutls_free( *state);
-		return GNUTLS_E_MEMORY_ERROR;
-	}
-
-	(*state)->gnutls_internals.resumed = RESUME_FALSE;
-
-	(*state)->gnutls_internals.expire_time = DEFAULT_EXPIRE_TIME; /* one hour default */
-
-	gnutls_transport_set_lowat((*state), DEFAULT_LOWAT); /* the default for tcp */
-
-	gnutls_handshake_set_max_data_buffer_size( (*state), MAX_HANDSHAKE_DATA_BUFFER_SIZE);
-
-	/* Allocate a minimum size for recv_data 
-	 * This is allocated in order to avoid small messages, makeing
-	 * the receive procedure slow.
-	 */
-	(*state)->gnutls_internals.recv_buffer.data = gnutls_malloc(INITIAL_RECV_BUFFER_SIZE);
-	
-	/* set the default maximum record size for TLS
-	 */
-	(*state)->security_parameters.max_record_size = DEFAULT_MAX_RECORD_SIZE;
-	(*state)->gnutls_internals.proposed_record_size = DEFAULT_MAX_RECORD_SIZE;
-
-	/* everything else not initialized here is initialized
-	 * as NULL or 0. This is why calloc is used.
-	 */
-
-	return 0;
-}
-
-/**
-  * gnutls_deinit - This function clears all buffers associated with the &state
-  * @state: is a &GNUTLS_STATE structure.
-  *
-  * This function clears all buffers associated with the &state.
-  **/
-void gnutls_deinit(GNUTLS_STATE state)
-{
-	/* if the session has failed abnormally it has to be removed from the db */
-	if ( state->gnutls_internals.resumable==RESUME_FALSE) {
-		_gnutls_db_remove_session( state, state->security_parameters.session_id, state->security_parameters.session_id_size);
-	}
-
-	/* remove auth info firstly */
-	_gnutls_free_auth_info(state );
-
-#ifdef HAVE_LIBGDBM
-	/* close the database - resuming sessions */
-	if ( state->gnutls_internals.db_reader != NULL)
-		gdbm_close(state->gnutls_internals.db_reader);
-#endif
-
-	_gnutls_clear_handshake_buffers( state);
-
-	gnutls_sfree_datum(&state->connection_state.read_mac_secret);
-	gnutls_sfree_datum(&state->connection_state.write_mac_secret);
-
-	_gnutls_free(state->gnutls_internals.recv_buffer.data);
-	_gnutls_free(state->gnutls_internals.buffer.data);
-	_gnutls_free(state->gnutls_internals.buffer_handshake.data);
-	_gnutls_free(state->gnutls_internals.hash_buffer.data);
-	_gnutls_free(state->gnutls_internals.send_buffer.data);
-
-	gnutls_clear_creds( state);
-
-	if (state->connection_state.read_cipher_state != NULL)
-		gnutls_cipher_deinit(state->connection_state.read_cipher_state);
-	if (state->connection_state.write_cipher_state != NULL)
-		gnutls_cipher_deinit(state->connection_state.write_cipher_state);
-
-	gnutls_sfree_datum( &state->cipher_specs.server_write_mac_secret);
-	gnutls_sfree_datum( &state->cipher_specs.client_write_mac_secret);
-	gnutls_sfree_datum( &state->cipher_specs.server_write_IV);
-	gnutls_sfree_datum( &state->cipher_specs.client_write_IV);
-	gnutls_sfree_datum( &state->cipher_specs.server_write_key);
-	gnutls_sfree_datum( &state->cipher_specs.client_write_key);
-
-	if (state->gnutls_key != NULL) {
-		_gnutls_mpi_release(&state->gnutls_key->KEY);
-		_gnutls_mpi_release(&state->gnutls_key->client_Y);
-		_gnutls_mpi_release(&state->gnutls_key->client_p);
-		_gnutls_mpi_release(&state->gnutls_key->client_g);
-
-		_gnutls_mpi_release(&state->gnutls_key->u);
-		_gnutls_mpi_release(&state->gnutls_key->a);
-		_gnutls_mpi_release(&state->gnutls_key->x);
-		_gnutls_mpi_release(&state->gnutls_key->A);
-		_gnutls_mpi_release(&state->gnutls_key->B);
-		_gnutls_mpi_release(&state->gnutls_key->b);
-
-		_gnutls_mpi_release(&state->gnutls_key->dh_secret);
-		_gnutls_free(state->gnutls_key);
-
-		state->gnutls_key = NULL;
-	}
-
-	/* free priorities */
-	_gnutls_free(state->gnutls_internals.MACAlgorithmPriority.algorithm_priority);
-	_gnutls_free(state->gnutls_internals.ProtocolPriority.algorithm_priority);
-	_gnutls_free(state->gnutls_internals.KXAlgorithmPriority.algorithm_priority);
-	_gnutls_free(state->gnutls_internals.BulkCipherAlgorithmPriority.algorithm_priority);
-	_gnutls_free(state->gnutls_internals.CompressionMethodPriority.algorithm_priority);
-
-	_gnutls_free(state->gnutls_internals.db_name);
-
-	memset( state, 0, sizeof(struct GNUTLS_STATE_INT));
-	gnutls_free(state);
-
-	return;
-}
-
 inline
 static void _gnutls_cal_PRF_A( MACAlgorithm algorithm, void *secret, int secret_size, void *seed, int seed_size, void* result)
 {
@@ -412,7 +263,7 @@ int gnutls_PRF( opaque * secret, int secret_size, uint8 * label, int label_size,
   * This function may also return GNUTLS_E_AGAIN, or GNUTLS_E_INTERRUPTED.
   *
   **/
-int gnutls_bye( GNUTLS_STATE state, CloseRequest how)
+int gnutls_bye( GNUTLS_STATE state, GNUTLS_CloseRequest how)
 {
 	int ret = 0, ret2 = 0;
 
@@ -420,7 +271,7 @@ int gnutls_bye( GNUTLS_STATE state, CloseRequest how)
 		case STATE0:
 		case STATE60:
 			if (STATE==STATE60) {
-				ret = _gnutls_write_flush( state);
+				ret = _gnutls_io_write_flush( state);
 			} else {
 				ret = gnutls_alert_send( state, GNUTLS_AL_WARNING, GNUTLS_A_CLOSE_NOTIFY);
 				STATE = STATE60;
@@ -446,6 +297,27 @@ int gnutls_bye( GNUTLS_STATE state, CloseRequest how)
 	return 0;
 }
 
+inline
+static void _gnutls_session_invalidate( GNUTLS_STATE state) {
+	state->gnutls_internals.valid_connection = VALID_FALSE;
+}
+
+
+inline
+static void _gnutls_session_unresumable( GNUTLS_STATE state) {
+	state->gnutls_internals.resumable = RESUME_FALSE;
+}
+
+/* returns 0 if session is valid
+ */
+inline
+static int _gnutls_session_is_valid( GNUTLS_STATE state) {
+	if (state->gnutls_internals.valid_connection==VALID_FALSE)
+		return GNUTLS_E_INVALID_SESSION;
+	
+	return 0;
+}
+
 /* This function behave exactly like write(). The only difference is 
  * that it accepts, the gnutls_state and the ContentType of data to
  * send (if called by the user the Content is specific)
@@ -472,9 +344,10 @@ ssize_t gnutls_send_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 		return GNUTLS_E_INVALID_PARAMETERS;
 	}
 
-	if (state->gnutls_internals.valid_connection == VALID_FALSE || state->gnutls_internals.may_write != 0) {
-		return GNUTLS_E_INVALID_SESSION;
-	}
+	if (type!=GNUTLS_ALERT) /* alert messages are sent anyway */
+		if ( _gnutls_session_is_valid( state) || state->gnutls_internals.may_write != 0) {
+			return GNUTLS_E_INVALID_SESSION;
+		}
 
 	headers[0]=type;
 	
@@ -487,11 +360,8 @@ ssize_t gnutls_send_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 	headers[1]=_gnutls_version_get_major( lver);
 	headers[2]=_gnutls_version_get_minor( lver);
 
-	
-#ifdef RECORD_DEBUG
-	_gnutls_log( "Record: Sending Packet[%d] %s(%d) with length: %d\n",
+	_gnutls_record_log( "REC: Sending Packet[%d] %s(%d) with length: %d\n",
 		(int) uint64touint32(&state->connection_state.write_sequence_number), _gnutls_packet2str(type), type, sizeofdata);
-#endif
 
 	if ( sizeofdata > MAX_RECORD_SIZE)
 		data2send = MAX_RECORD_SIZE;
@@ -501,14 +371,14 @@ ssize_t gnutls_send_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 	/* Only encrypt if we don't have data to send 
 	 * from the previous run. - probably interrupted.
 	 */
-	if (state->gnutls_internals.send_buffer.size > 0) {
-		ret = _gnutls_write_flush( state);
+	if (state->gnutls_internals.record_send_buffer.size > 0) {
+		ret = _gnutls_io_write_flush( state);
 		if (ret > 0) cipher_size = ret;
 		else cipher_size = 0;
 		
 		cipher = NULL;
 
-		retval = state->gnutls_internals.send_buffer_user_size;
+		retval = state->gnutls_internals.record_send_buffer_user_size;
 	} else {
 		cipher_size = _gnutls_encrypt( state, headers, RECORD_HEADER_SIZE, data, data2send, &cipher, type);
 		if (cipher_size <= 0) {
@@ -518,19 +388,19 @@ ssize_t gnutls_send_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 		}
 
 		retval = data2send;
-		state->gnutls_internals.send_buffer_user_size =	data2send;
+		state->gnutls_internals.record_send_buffer_user_size =	data2send;
 
 		/* increase sequence number
 		 */
 		if (uint64pp( &state->connection_state.write_sequence_number) != 0) {
-			state->gnutls_internals.valid_connection = VALID_FALSE;
+			_gnutls_session_invalidate( state);
 			gnutls_assert();
 			/* FIXME: Somebody has to do rehandshake before that.
 			 */
 			return GNUTLS_E_RECORD_LIMIT_REACHED;
 		}
 
-		ret = _gnutls_write_buffered( state, cipher, cipher_size);
+		ret = _gnutls_io_write_buffered( state, cipher, cipher_size);
 	}
 
 	if ( ret != cipher_size) {
@@ -548,21 +418,18 @@ ssize_t gnutls_send_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 			ret = GNUTLS_E_UNKNOWN_ERROR;
 		}
 
-		state->gnutls_internals.valid_connection = VALID_FALSE;
-		state->gnutls_internals.resumable = RESUME_FALSE;
+		_gnutls_session_unresumable( state);
+		_gnutls_session_invalidate( state);
 		gnutls_assert();
 		return ret;
 	}
 
-	state->gnutls_internals.send_buffer_user_size = 0;
+	state->gnutls_internals.record_send_buffer_user_size = 0;
 
 	gnutls_free(cipher);
 
-#ifdef RECORD_DEBUG
-	_gnutls_log( "Record: Sent Packet[%d] %s(%d) with length: %d\n",
+	_gnutls_record_log( "REC: Sent Packet[%d] %s(%d) with length: %d\n",
 	(int) uint64touint32(&state->connection_state.write_sequence_number), _gnutls_packet2str(type), type, cipher_size);
-#endif
-
 
 	return retval;
 }
@@ -574,13 +441,12 @@ ssize_t _gnutls_send_change_cipher_spec( GNUTLS_STATE state, int again)
 {
 	opaque data[1] = { GNUTLS_TYPE_CHANGE_CIPHER_SPEC };
 
-#ifdef HANDSHAKE_DEBUG
-	_gnutls_log( "Record: Sent ChangeCipherSpec\n");
-#endif
+	_gnutls_handshake_log( "REC: Sent ChangeCipherSpec\n");
+
 	if (again==0)
 		return gnutls_send_int( state, GNUTLS_CHANGE_CIPHER_SPEC, -1, data, 1);
 	else {
-		return _gnutls_write_flush( state);
+		return _gnutls_io_write_flush( state);
 	}
 }
 
@@ -598,8 +464,193 @@ static int _gnutls_check_recv_type( ContentType recv_type) {
 
 }
 
+
+/* Checks if there are pending data into the record buffers. If there are
+ * then it copies the data.
+ */
+static int _gnutls_check_buffers( GNUTLS_STATE state, ContentType type, opaque* data, int sizeofdata) {
+	int ret = 0, ret2=0;
+	if ( (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE) && _gnutls_record_buffer_get_size(type, state) > 0) {
+		ret = _gnutls_record_buffer_get(type, state, data, sizeofdata);
+		if (ret < 0) {
+			gnutls_assert();
+			return ret;
+		}
+		
+		/* if the buffer just got empty */
+		if (_gnutls_record_buffer_get_size(type, state)==0) {
+			if ( (ret2=_gnutls_io_clear_peeked_data( state)) < 0) {
+				gnutls_assert();
+				return ret2;
+			}
+		}
+
+		return ret;
+	}
+	
+	return 0;
+}
+
+
 #define CHECK_RECORD_VERSION
 
+/* Checks the record headers and returns the length, version and
+ * content type.
+ */
+static int _gnutls_check_record_headers( GNUTLS_STATE state, uint8 headers[RECORD_HEADER_SIZE], ContentType type, 
+	HandshakeType htype, /*output*/ ContentType *recv_type, GNUTLS_Version *version, uint16 *length, uint16* header_size) {
+
+	/* Read the first two bytes to determine if this is a 
+	 * version 2 message 
+	 */
+
+	if ( htype == GNUTLS_CLIENT_HELLO && type==GNUTLS_HANDSHAKE && headers[0] > 127) { 
+
+		/* if msb set and expecting handshake message
+		 * it should be SSL 2 hello 
+		 */
+		*version = GNUTLS_VERSION_UNKNOWN; /* assume unknown version */
+		*length = (((headers[0] & 0x7f) << 8)) | headers[1];
+
+		/* SSL 2.0 headers */
+		*header_size = 2;
+		*recv_type = GNUTLS_HANDSHAKE; /* we accept only v2 client hello
+					       */
+		
+		/* in order to assist the handshake protocol.
+		 * V2 compatibility is a mess.
+		 */
+		state->gnutls_internals.v2_hello = *length;
+
+		_gnutls_record_log( "REC: V2 packet received. Length: %d\n", *length);
+
+	} else {
+		/* version 3.x 
+		 */
+		*recv_type = headers[0];
+#ifdef CHECK_RECORD_VERSION
+		*version = _gnutls_version_get( headers[1], headers[2]);
+#endif
+
+		*length = READuint16( &headers[3]);
+	}
+
+	return 0;
+}
+
+/* Here we check if the advertized version is the one we
+ * negotiated in the handshake.
+ */
+inline
+static int _gnutls_check_record_version( GNUTLS_STATE state, HandshakeType htype, GNUTLS_Version version) 
+{
+#ifdef CHECK_RECORD_VERSION
+	if ( (htype!=GNUTLS_CLIENT_HELLO && htype!=GNUTLS_SERVER_HELLO) && gnutls_protocol_get_version(state) != version) {
+		gnutls_assert();
+
+		_gnutls_record_log( "REC: INVALID VERSION PACKET: (%d) %d.%d\n", htype, _gnutls_version_get_major(version), _gnutls_version_get_minor(version));
+
+		return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+	}
+#endif
+
+	return 0;
+}
+
+/* This function will check if the received record type is
+ * the one we actually expecting for.
+ */
+static int _gnutls_record_check_type( GNUTLS_STATE state, ContentType recv_type,
+	ContentType type, HandshakeType htype, opaque* data, int data_size) {
+	
+	int ret;
+
+	if ( (recv_type == type) && (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE)) {
+		_gnutls_record_buffer_put(type, state, (void *) data, data_size);
+	} else {
+		switch (recv_type) {
+		case GNUTLS_ALERT:
+
+			_gnutls_record_log( "REC: Alert[%d|%d] - %s - was received\n", data[0], data[1], gnutls_alert_get_name((int)data[1]));
+
+			state->gnutls_internals.last_alert = data[1];
+
+			/* if close notify is received and
+			 * the alert is not fatal
+			 */
+			if (data[1] == GNUTLS_A_CLOSE_NOTIFY && data[0] != GNUTLS_AL_FATAL) {
+				/* If we have been expecting for an alert do 
+				 */
+
+				return GNUTLS_E_INT_RET_0; /* EOF */
+			} else {
+			
+				/* if the alert is FATAL or WARNING
+				 * return the apropriate message
+				 */
+				
+				gnutls_assert();		
+				ret = GNUTLS_E_WARNING_ALERT_RECEIVED;
+				if (data[0] == GNUTLS_AL_FATAL) {
+					_gnutls_session_unresumable( state);
+					_gnutls_session_invalidate( state);
+
+					ret = GNUTLS_E_FATAL_ALERT_RECEIVED;
+				}
+
+				return ret;
+			}
+			break;
+
+		case GNUTLS_CHANGE_CIPHER_SPEC:
+			/* this packet is now handled above */
+			gnutls_assert();
+	
+			return GNUTLS_E_UNEXPECTED_PACKET;
+
+		case GNUTLS_APPLICATION_DATA:
+			/* even if data is unexpected put it into the buffer */
+			if ( (ret=_gnutls_record_buffer_put(recv_type, state, (void *) data, data_size)) < 0) {
+				gnutls_assert();
+				return ret;
+			}
+
+			gnutls_assert();
+			
+			/* the got_application data is only returned
+			 * if expecting client hello (for rehandshake
+			 * reasons). Otherwise it is an unexpected packet
+			 */
+			if (htype == GNUTLS_CLIENT_HELLO && type==GNUTLS_HANDSHAKE)
+				return GNUTLS_E_GOT_APPLICATION_DATA;
+			else return GNUTLS_E_UNEXPECTED_PACKET;
+			
+			break;
+		case GNUTLS_HANDSHAKE:
+			/* This is only legal if HELLO_REQUEST is received - and we are a client 
+			 */
+			if ( state->security_parameters.entity==GNUTLS_SERVER) {
+				gnutls_assert();
+				return GNUTLS_E_UNEXPECTED_PACKET;
+			}
+			gnutls_assert();
+			
+			return _gnutls_recv_hello_request( state, data, data_size);
+
+			break;
+		default:
+
+			_gnutls_record_log( "REC: Received Unknown packet %d expecting %d\n", recv_type, type);
+
+			gnutls_assert();
+			return GNUTLS_E_UNKNOWN_ERROR;
+		}
+	}
+	
+	return 0;
+
+}
+
 /* This function behave exactly like read(). The only difference is 
  * that it accepts, the gnutls_state and the ContentType of data to
  * send (if called by the user the Content is Userdata only)
@@ -616,10 +667,10 @@ ssize_t gnutls_recv_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 	uint8 *ciphertext;
 	uint8 *recv_data;
 	int ret, ret2;
-	int header_size;
-
-	begin:
+	uint16 header_size;
 
+	/* default headers for TLS 1.0
+	 */
 	header_size = RECORD_HEADER_SIZE;
 	ret = 0;
 
@@ -627,7 +678,7 @@ ssize_t gnutls_recv_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 		return GNUTLS_E_INVALID_PARAMETERS;
 	}
 
-	if (state->gnutls_internals.valid_connection == VALID_FALSE || state->gnutls_internals.may_read!=0) {
+	if ( _gnutls_session_is_valid(state)!=0 || state->gnutls_internals.may_read!=0) {
 		gnutls_assert();
 		return GNUTLS_E_INVALID_SESSION;
 	}
@@ -635,74 +686,34 @@ ssize_t gnutls_recv_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 	/* If we have enough data in the cache do not bother receiving
 	 * a new packet. (in order to flush the cache)
 	 */
-	if ( (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE) && gnutls_get_data_buffer_size(type, state) > 0) {
-		ret = gnutls_get_data_buffer(type, state, data, sizeofdata);
-		if (ret < 0) {
-			gnutls_assert();
-			return ret;
-		}
-		
-		/* if the buffer just got empty */
-		if (gnutls_get_data_buffer_size(type, state)==0) {
-			if ( (ret2=_gnutls_clear_peeked_data( state)) < 0) {
-				gnutls_assert();
-				return ret2;
-			}
-		}
-
+	ret = _gnutls_check_buffers( state, type, data, sizeofdata);
+	if (ret != 0)
 		return ret;
-	}
-	
-	/* in order for GNUTLS_E_AGAIN to be returned the socket
-	 * must be set to non blocking mode
-	 */
-	if ( (ret = _gnutls_read_buffered( state, &headers, header_size, -1)) != header_size) {
+
+
+	if ( (ret = _gnutls_io_read_buffered( state, &headers, header_size, -1)) != header_size) {
 		if (ret < 0 && gnutls_error_is_fatal(ret)==0) return ret;
 
-		state->gnutls_internals.valid_connection = VALID_FALSE;
+		_gnutls_session_invalidate( state);
 		if (type==GNUTLS_ALERT) {
 			gnutls_assert();
 			return 0; /* we were expecting close notify */
 		}
-		state->gnutls_internals.resumable = RESUME_FALSE;
+		_gnutls_session_unresumable( state);
 		gnutls_assert();
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 	}
 
-	/* Read the first two bytes to determine if this is a 
-	 * version 2 message 
-	 */
-	if ( htype == GNUTLS_CLIENT_HELLO && type==GNUTLS_HANDSHAKE && headers[0] > 127) { 
-
-	/* if msb set and expecting handshake message
-	 * it should be SSL 2 hello 
-	 */
-		version = GNUTLS_VERSION_UNKNOWN; /* assume unknown version */
-		length = (((headers[0] & 0x7f) << 8)) | headers[1];
-
-		header_size = 2;
-		recv_type = GNUTLS_HANDSHAKE; /* we accept only v2 client hello
-					       */
-		state->gnutls_internals.v2_hello = length;
-#ifdef RECORD_DEBUG
-		_gnutls_log( "Record: V2 packet received. Length: %d\n", length);
-#endif
-
-	} else {
-		/* version 3.x 
-		 */
-		recv_type = headers[0];
-#ifdef CHECK_RECORD_VERSION
-		version = _gnutls_version_get( headers[1], headers[2]);
-#endif
-
-		length = READuint16( &headers[3]);
+	if ( (ret=_gnutls_check_record_headers( state, headers, type, htype, &recv_type, &version, &length, &header_size)) < 0) {
+		gnutls_assert();
+		return ret;
 	}
 
 	/* Here we check if the Type of the received packet is
 	 * ok. 
 	 */
 	if ( (ret = _gnutls_check_recv_type( recv_type)) < 0) {
+		
 		gnutls_assert();
 		return ret;
 	}
@@ -710,41 +721,34 @@ ssize_t gnutls_recv_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 	/* Here we check if the advertized version is the one we
 	 * negotiated in the handshake.
 	 */
-#ifdef CHECK_RECORD_VERSION
-	if ( (htype!=GNUTLS_CLIENT_HELLO && htype!=GNUTLS_SERVER_HELLO) && gnutls_protocol_get_version(state) != version) {
+	if ( (ret=_gnutls_check_record_version( state, htype, version)) < 0) {
 		gnutls_assert();
-# ifdef RECORD_DEBUG
-		_gnutls_log( "Record: INVALID VERSION PACKET: (%d/%d) %d.%d\n", headers[0], htype, headers[1], headers[2]);
-# endif
-		state->gnutls_internals.resumable = RESUME_FALSE;
-		return GNUTLS_E_UNSUPPORTED_VERSION_PACKET;
+		_gnutls_session_invalidate( state);
+		return ret;
 	}
-#endif
 
-#ifdef RECORD_DEBUG
-	_gnutls_log( "Record: Expected Packet[%d] %s(%d) with length: %d\n",
+	_gnutls_record_log( "REC: Expected Packet[%d] %s(%d) with length: %d\n",
 		(int) uint64touint32(&state->connection_state.read_sequence_number), _gnutls_packet2str(type), type, sizeofdata);
-	_gnutls_log( "Record: Received Packet[%d] %s(%d) with length: %d\n",
+	_gnutls_record_log( "REC: Received Packet[%d] %s(%d) with length: %d\n",
 		(int) uint64touint32(&state->connection_state.read_sequence_number), _gnutls_packet2str(recv_type), recv_type, length);
-#endif
 
 	if (length > MAX_RECV_SIZE) {
-#ifdef RECORD_DEBUG
-		_gnutls_log( "Record: FATAL ERROR: Received packet with length: %d\n", length);
-#endif
-		state->gnutls_internals.valid_connection = VALID_FALSE;
-		state->gnutls_internals.resumable = RESUME_FALSE;
+
+		_gnutls_record_log( "REC: FATAL ERROR: Received packet with length: %d\n", length);
+
+		_gnutls_session_unresumable( state);
+		_gnutls_session_invalidate( state);
 		gnutls_assert();
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
 	}
 
 	/* check if we have that data into buffer. 
  	 */
-	if ( (ret = _gnutls_read_buffered( state, &recv_data, header_size+length, recv_type)) != length+header_size) {
+	if ( (ret = _gnutls_io_read_buffered( state, &recv_data, header_size+length, recv_type)) != length+header_size) {
 		if (ret<0 && gnutls_error_is_fatal(ret)==0) return ret;
 
-		state->gnutls_internals.valid_connection = VALID_FALSE;
-		state->gnutls_internals.resumable = RESUME_FALSE;
+		_gnutls_session_unresumable( state);
+		_gnutls_session_invalidate( state);
 		gnutls_assert();
 		return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;		
 	}
@@ -752,15 +756,15 @@ ssize_t gnutls_recv_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 /* ok now we are sure that we can read all the data - so
  * move on !
  */
-	_gnutls_clear_read_buffer( state);
+	_gnutls_io_clear_read_buffer( state);
 	ciphertext = &recv_data[header_size];
 	
 	/* decrypt the data we got
 	 */
 	tmplen = _gnutls_decrypt( state, ciphertext, length, &tmpdata, recv_type);
 	if (tmplen < 0) {
-		state->gnutls_internals.valid_connection = VALID_FALSE;
-		state->gnutls_internals.resumable = RESUME_FALSE;
+		_gnutls_session_unresumable( state);
+		_gnutls_session_invalidate( state);
 		gnutls_assert();
 		return tmplen;
 	}
@@ -768,9 +772,8 @@ ssize_t gnutls_recv_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 	/* Check if this is a CHANGE_CIPHER_SPEC
 	 */
 	if (type == GNUTLS_CHANGE_CIPHER_SPEC && recv_type == GNUTLS_CHANGE_CIPHER_SPEC) {
-#ifdef RECORD_DEBUG
-		_gnutls_log( "Record: ChangeCipherSpec Packet was received\n");
-#endif
+
+		_gnutls_record_log( "REC: ChangeCipherSpec Packet was received\n");
 
 		if (tmplen!=sizeofdata) { /* sizeofdata should be 1 */
 			gnutls_assert();
@@ -783,179 +786,54 @@ ssize_t gnutls_recv_int( GNUTLS_STATE state, ContentType type, HandshakeType hty
 		return tmplen;
 	}
 
-#ifdef RECORD_DEBUG
-	_gnutls_log( "Record: Decrypted Packet[%d] %s(%d) with length: %d\n",
+	_gnutls_record_log( "REC: Decrypted Packet[%d] %s(%d) with length: %d\n",
 		(int) uint64touint32(&state->connection_state.read_sequence_number), _gnutls_packet2str(recv_type), recv_type, tmplen);
-#endif
 
 	/* increase sequence number */
 	if (uint64pp( &state->connection_state.read_sequence_number)!=0) {
-		state->gnutls_internals.valid_connection = VALID_FALSE;
+		_gnutls_session_invalidate( state);
 		gnutls_free(tmpdata);
 		gnutls_assert();
 		return GNUTLS_E_RECORD_LIMIT_REACHED;
 	}
 
-	if ( (recv_type == type) && (type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE)) {
-		gnutls_insert_to_data_buffer(type, state, (void *) tmpdata, tmplen);
-	} else {
-		switch (recv_type) {
-		case GNUTLS_ALERT:
-#ifdef RECORD_DEBUG
-			_gnutls_log( "Record: Alert[%d|%d] - %s - was received\n", tmpdata[0], tmpdata[1], _gnutls_alert2str((int)tmpdata[1]));
-#endif
-			state->gnutls_internals.last_alert = tmpdata[1];
+	if ( (ret=_gnutls_record_check_type( state, recv_type, type, htype, tmpdata, tmplen)) < 0) {
+		gnutls_free( tmpdata);
 
-			/* if close notify is received and
-			 * the alert is not fatal
-			 */
-			if (tmpdata[1] == GNUTLS_A_CLOSE_NOTIFY && tmpdata[0] != GNUTLS_AL_FATAL) {
-				/* If we have been expecting for an alert do 
-				 * not call close().
-				 */
-				if (type != GNUTLS_ALERT) 
-					do ret=gnutls_bye( state, GNUTLS_SHUT_WR);
-					while(ret==GNUTLS_E_INTERRUPTED || ret==GNUTLS_E_AGAIN);
-				gnutls_free(tmpdata);
+		if (ret==GNUTLS_E_INT_RET_0) return 0;
 
-				return 0; /* EOF */
-			} else {
-			
-				/* if the alert is FATAL or WARNING
-				 * return the apropriate message
-				 */
-			
-				ret = GNUTLS_E_WARNING_ALERT_RECEIVED;
-				if (tmpdata[0] == GNUTLS_AL_FATAL) {
-					state->gnutls_internals.valid_connection = VALID_FALSE;
-					state->gnutls_internals.resumable = RESUME_FALSE;
-					
-					ret = GNUTLS_E_FATAL_ALERT_RECEIVED;
-				}
-
-				gnutls_free(tmpdata);
-
-				return ret;
-			}
-			break;
-
-		case GNUTLS_CHANGE_CIPHER_SPEC:
-			/* this packet is now handled above */
-			gnutls_assert();
-	
-			gnutls_free(tmpdata);
-
-			return GNUTLS_E_UNEXPECTED_PACKET;
-		case GNUTLS_APPLICATION_DATA:
-			/* even if data is unexpected put it into the buffer */
-			if ( (ret=gnutls_insert_to_data_buffer(recv_type, state, (void *) tmpdata, tmplen)) < 0) {
-				gnutls_assert();
-				return ret;
-			}
-
-			gnutls_assert();
-			gnutls_free(tmpdata);
-			
-			goto begin; /* ok we received the packet, 
-			             * and now we should get the one
-			             * we expected.
-			             */
-			
-			break;
-		case GNUTLS_HANDSHAKE:
-			/* This is only legal if HELLO_REQUEST is received - and we are a client 
-			 */
-			if (htype!=GNUTLS_HELLO_REQUEST && state->security_parameters.entity==GNUTLS_SERVER) {
-				gnutls_assert();
-				gnutls_free( tmpdata);
-				return GNUTLS_E_UNEXPECTED_PACKET;
-			}
-
-			break;
-		default:
-#ifdef RECORD_DEBUG
-			_gnutls_log( "Record: Received Unknown packet %d expecting %d\n", recv_type, type);
-#endif
-			gnutls_assert();
-			return GNUTLS_E_UNKNOWN_ERROR;
-		}
+		gnutls_assert();
+		return ret;
 	}
-
+	gnutls_free( tmpdata);
 
 	/* Get Application data from buffer */
 	if ((type == GNUTLS_APPLICATION_DATA || type == GNUTLS_HANDSHAKE) && (recv_type == type)) {
-		ret = gnutls_get_data_buffer(type, state, data, sizeofdata);
+
+		ret = _gnutls_record_buffer_get(type, state, data, sizeofdata);
 		if (ret < 0) {
 			gnutls_assert();
 			return ret;
 		}
 
 		/* if the buffer just got empty */
-		if (gnutls_get_data_buffer_size(type, state)==0) {
-			if ( (ret2 = _gnutls_clear_peeked_data( state)) < 0) {
+		if (_gnutls_record_buffer_get_size(type, state)==0) {
+			if ( (ret2 = _gnutls_io_clear_peeked_data( state)) < 0) {
 				gnutls_assert();
 				return ret2;
 			}
 		}
 
-		gnutls_free(tmpdata);
 	} else {
-		if (recv_type == GNUTLS_HANDSHAKE) {
-			/* we may get a hello request */
-			ret = _gnutls_recv_hello_request( state, tmpdata, tmplen);
-		} else {
-			gnutls_assert();
-			ret = GNUTLS_E_UNEXPECTED_PACKET; 
-				/* we didn't get what we wanted to 
-				 */
-			if (recv_type == GNUTLS_APPLICATION_DATA)
-				ret = GNUTLS_E_GOT_APPLICATION_DATA;
-		}
-		gnutls_free(tmpdata);
+		gnutls_assert();
+		ret = GNUTLS_E_UNEXPECTED_PACKET; 
+		/* we didn't get what we wanted to 
+		 */
 	}
 
 	return ret;
 }
 
-/**
-  * gnutls_cipher_get_algo - Returns the currently used cipher.
-  * @state: is a &GNUTLS_STATE structure.
-  *
-  * Returns the currently used cipher.
-  **/
-BulkCipherAlgorithm gnutls_cipher_get_algo( GNUTLS_STATE state) {
-	return state->security_parameters.read_bulk_cipher_algorithm;
-}
-
-/**
-  * gnutls_kx_get_algo - Returns the key exchange algorithm.
-  * @state: is a &GNUTLS_STATE structure.
-  *
-  * Returns the key exchange algorithm used in the last handshake.
-  **/
-KXAlgorithm gnutls_kx_get_algo( GNUTLS_STATE state) {
-	return state->security_parameters.kx_algorithm;
-}
-
-/**
-  * gnutls_mac_get_algo - Returns the currently used mac algorithm.
-  * @state: is a &GNUTLS_STATE structure.
-  *
-  * Returns the currently used mac algorithm.
-  **/
-MACAlgorithm gnutls_mac_get_algo( GNUTLS_STATE state) {
-	return state->security_parameters.read_mac_algorithm;
-}
-
-/**
-  * gnutls_compression_get_algo - Returns the currently used compression algorithm.
-  * @state: is a &GNUTLS_STATE structure.
-  *
-  * Returns the currently used compression method.
-  **/
-CompressionMethod gnutls_compression_get_algo( GNUTLS_STATE state) {
-	return state->security_parameters.read_compression_algorithm;
-}
 
 /* Taken from libgcrypt */
 
@@ -1031,7 +909,7 @@ gnutls_check_version( const char *req_version )
 
 
 /**
-  * gnutls_write - sends to the peer the specified data
+  * gnutls_record_send - sends to the peer the specified data
   * @state: is a &GNUTLS_STATE structure.
   * @data: contains the data to send
   * @sizeofdata: is the length of the data
@@ -1049,12 +927,12 @@ gnutls_check_version( const char *req_version )
   * Returns the number of bytes sent, or a negative error code.
   *
   **/
-ssize_t gnutls_write( GNUTLS_STATE state, const void *data, size_t sizeofdata) {
+ssize_t gnutls_record_send( GNUTLS_STATE state, const void *data, size_t sizeofdata) {
 	return gnutls_send_int( state, GNUTLS_APPLICATION_DATA, -1, data, sizeofdata);
 }
 
 /**
-  * gnutls_read - reads data from the TLS connection
+  * gnutls_record_recv - reads data from the TLS record protocol
   * @state: is a &GNUTLS_STATE structure.
   * @data: contains the data to send
   * @sizeofdata: is the length of the data
@@ -1064,12 +942,12 @@ ssize_t gnutls_write( GNUTLS_STATE state, const void *data, size_t sizeofdata) {
   * Also returns the number of bytes received, zero on EOF, but
   * a negative error code in case of an error.
   *
-  * If this function returns GNUTLS_E_REHANDSHAKE, then you must
-  * either send an alert containing NO_RENEGOTIATION, or perform a
-  * handshake. (only a client may receive this message)
+  * If this function returns GNUTLS_E_REHANDSHAKE, then you may
+  * ignore this message, send an alert containing NO_RENEGOTIATION, 
+  * or perform a handshake again. (only a client may receive this message)
   *
   **/
-ssize_t gnutls_read( GNUTLS_STATE state, void *data, size_t sizeofdata) {
+ssize_t gnutls_record_recv( GNUTLS_STATE state, void *data, size_t sizeofdata) {
 	return gnutls_recv_int( state, GNUTLS_APPLICATION_DATA, -1, data, sizeofdata);
 }
 
@@ -1105,8 +983,8 @@ size_t gnutls_record_get_max_size( GNUTLS_STATE state) {
   * Not all TLS implementations use or even understand this extension.
   *
   **/
-size_t gnutls_record_set_max_size( GNUTLS_STATE state, size_t size) {
-size_t new_size;
+ssize_t gnutls_record_set_max_size( GNUTLS_STATE state, size_t size) {
+ssize_t new_size;
 
 	if (state->security_parameters.entity==GNUTLS_SERVER)
 		return GNUTLS_E_INVALID_REQUEST;
