@@ -3,6 +3,8 @@
 
 #include <gnutls_int.h>
 
+void _gnutls_record_set_default_version(GNUTLS_STATE state, GNUTLS_Version version);
+
 void _gnutls_state_cert_type_set( GNUTLS_STATE state, CertificateType);
 KXAlgorithm gnutls_kx_get( GNUTLS_STATE state);
 CertificateType gnutls_cert_type_get( GNUTLS_STATE state);
@@ -20,6 +22,7 @@ int _gnutls_dh_set_secret_bits( GNUTLS_STATE state, int bits);
 int _gnutls_dh_set_prime_bits( GNUTLS_STATE state, int bits);
 int _gnutls_dh_get_prime_bits( GNUTLS_STATE state);
 void gnutls_dh_set_prime_bits( GNUTLS_STATE state, int bits);
+void _gnutls_handshake_internal_state_clear( GNUTLS_STATE);
 
 int _gnutls_openpgp_send_fingerprint( GNUTLS_STATE state);
 
