@@ -29,7 +29,7 @@
 
 #include "x509_asn1.h" 
 #include "x509_der.h"
-
+#include <gnutls_str.h>
 
 
 /* define used for visiting trees */
@@ -58,7 +58,8 @@ _asn1_add_node(unsigned int type)
   if(parse_mode==PARSE_MODE_CHECK) return NULL;
 
   punt=(node_asn *) gnutls_malloc(sizeof(node_asn));
-
+  if (punt==NULL) return NULL;
+  
   punt->left=NULL;
   punt->name=NULL;
   punt->type=type; 
@@ -91,6 +92,8 @@ _asn1_set_value(node_asn *node,unsigned char *value,unsigned int len)
   }
   if(!len) return node;
   node->value=(unsigned char *) gnutls_malloc(len);
+  if (node->value==NULL) return NULL;
+  
   memcpy(node->value,value,len);
   return node;
 }
@@ -122,7 +125,9 @@ _asn1_set_name(node_asn *node,char *name)
   if(strlen(name))
 	{
   	node->name=(char *) gnutls_malloc(strlen(name)+1);
-  	strcpy(node->name,name);
+  	if (node->name==NULL) return NULL;
+  	/* this strcpy is checked */
+  	strcpy(node->name, name);
       }
   else node->name=NULL;
   return node;
@@ -306,7 +311,7 @@ _asn1_find_node(node_asn *pointer,char *name)
     n_start++;
   }
   else{
-    strcpy(n,n_start);
+    _gnutls_str_cpy(n,sizeof(n),n_start);
     n_start=NULL;
   }
 
@@ -327,7 +332,7 @@ _asn1_find_node(node_asn *pointer,char *name)
       n_start++;
     }
     else{
-      strcpy(n,n_start);
+      _gnutls_str_cpy(n,sizeof(n),n_start);
       n_start=NULL;
     }
 
@@ -532,14 +537,14 @@ _asn1_create_static_structure(node_asn *pointer,char *file_name, char* out_name)
 
   memcpy(structure_name,slash_p,dot_p-slash_p);
   structure_name[dot_p-slash_p]=0;
-  strcat(structure_name,"_asn1_tab");
+  _gnutls_str_cat(structure_name, sizeof(structure_name),"_asn1_tab");
 
   if (out_name==NULL) {
 	  memcpy(file_out_name,file_name,dot_p-file_name);
 	  file_out_name[dot_p-file_name]=0;
-	  strcat(file_out_name,"_asn1_tab.c");
+	  _gnutls_str_cat(file_out_name, sizeof(file_out_name), "_asn1_tab.c");
   } else {
-  	  strncpy( file_out_name, out_name, sizeof(file_out_name));
+  	  _gnutls_str_cpy( file_out_name, sizeof(file_out_name), out_name);
   }
   file=fopen( file_out_name,"w");
 
@@ -951,7 +956,7 @@ asn1_create_structure(node_asn *root,char *source_name,node_asn **pointer,char *
     n[end-source_name]=0;
   }
   else{
-    strcpy(n,source_name);
+    _gnutls_str_cpy(n,sizeof(n),source_name);
   }
 
   res=_asn1_expand_identifier(&dest_node,root);
@@ -967,7 +972,7 @@ int
 _asn1_append_sequence_set(node_asn *node)
 {
   node_asn *p,*p2;
-  char *temp;
+  char temp[10];
   long n;
 
   if(!node || !(node->down)) return ASN_GENERIC_ERROR;
@@ -977,8 +982,8 @@ _asn1_append_sequence_set(node_asn *node)
   p2=_asn1_copy_structure3(p);
   while(p->right) p=p->right;
   _asn1_set_right(p,p2);
-  temp=(char *) gnutls_malloc(10);
-  if(p->name==NULL) strcpy(temp,"?1");
+  
+  if(p->name==NULL) _gnutls_str_cpy(temp,sizeof(temp),"?1");
   else{
     n=strtol(p->name+1,NULL,0);
     n++;
@@ -986,7 +991,6 @@ _asn1_append_sequence_set(node_asn *node)
     _asn1_ltostr(n,temp+1);
   } 
   _asn1_set_name(p2,temp);
-  gnutls_free(temp);
 
   return ASN_OK;
 }
@@ -1104,6 +1108,8 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
     if(len==0){
       if(isdigit(value[0])){
 	value_temp=(unsigned char *)gnutls_malloc(4);
+	if (value_temp==NULL) return ASN_MEM_ERROR;
+
 	_asn1_convert_integer(value,value_temp,4, &len);
       }
       else{ /* is an identifier like v1 */
@@ -1113,6 +1119,8 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
 	  if(type_field(p->type)==TYPE_CONSTANT){
 	    if((p->name) && (!strcmp(p->name,value))){
 	      value_temp=(unsigned char *)gnutls_malloc(4);
+	      if (value_temp==NULL) return ASN_MEM_ERROR;
+
 	      _asn1_convert_integer(p->value,value_temp,4, &len);
 	      break;
 	    }
@@ -1124,6 +1132,7 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
     }
     else{
       value_temp=(unsigned char *)gnutls_malloc(len);
+      if (value_temp==NULL) return ASN_MEM_ERROR;
       memcpy(value_temp,value,len);
     }
 
@@ -1143,6 +1152,8 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
 
     _asn1_length_der(len-k,NULL,&len2);
     temp=(unsigned char *)gnutls_malloc(len-k+len2);
+    if (temp==NULL) return ASN_MEM_ERROR;
+
     _asn1_octet_der(value_temp+k,len-k,temp,&len2);
     _asn1_set_value(node,temp,len2);
 
@@ -1153,6 +1164,8 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
       while(type_field(p->type)!=TYPE_DEFAULT) p=p->right;
       if(isdigit(p->value[0])){
 	default_temp=(unsigned char *)gnutls_malloc(4);
+        if (default_temp==NULL) return ASN_MEM_ERROR;
+
 	_asn1_convert_integer(p->value,default_temp,4,&len2);
       }
       else{ /* is an identifier like v1 */
@@ -1162,6 +1175,8 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
 	  if(type_field(p2->type)==TYPE_CONSTANT){
 	    if((p2->name) && (!strcmp(p2->name,p->value))){
 	      default_temp=(unsigned char *)gnutls_malloc(4);
+	      if (default_temp==NULL) return ASN_MEM_ERROR;
+
 	      _asn1_convert_integer(p2->value,default_temp,4,&len2);
 	      break;
 	    }
@@ -1225,6 +1240,8 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
   case  TYPE_OCTET_STRING:
     _asn1_length_der(len,NULL,&len2);
     temp=(unsigned char *)gnutls_malloc(len+len2);
+    if (temp==NULL) return ASN_MEM_ERROR;
+
     _asn1_octet_der(value,len,temp,&len2);
     _asn1_set_value(node,temp,len2);
     gnutls_free(temp);
@@ -1232,6 +1249,8 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
   case  TYPE_BIT_STRING:
     _asn1_length_der((len>>3)+2,NULL,&len2);
     temp=(unsigned char *)gnutls_malloc((len>>3)+2+len2);
+    if (temp==NULL) return ASN_MEM_ERROR;
+
     _asn1_bit_der(value,len,temp,&len2);
     _asn1_set_value(node,temp,len2);
     gnutls_free(temp);
@@ -1254,6 +1273,8 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
   case TYPE_ANY:
     _asn1_length_der(len,NULL,&len2);
     temp=(unsigned char *)gnutls_malloc(len+len2);
+    if (temp==NULL) return ASN_MEM_ERROR;
+
     _asn1_octet_der(value,len,temp,&len2);
     _asn1_set_value(node,temp,len2);
     gnutls_free(temp);
@@ -1271,36 +1292,39 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
 }
 
 #define PUT_VALUE( ptr, ptr_size, data, data_size) \
+	*len = data_size; \
 	if (ptr_size < data_size) { \
 		return ASN_MEM_ERROR; \
 	} else { \
 		memcpy( ptr, data, data_size); \
-		*len = data_size; \
 	}
-		
+
 #define PUT_STR_VALUE( ptr, ptr_size, data) \
-	if (ptr_size <= strlen(data)) { \
+	*len = strlen(data) + 1; \
+	if (ptr_size < *len) { \
 		return ASN_MEM_ERROR; \
 	} else { \
-		strcpy( ptr, data); \
-		*len = strlen(ptr)+1; \
+		/* this strcpy is checked */ \
+		strcpy(ptr, data); \
 	}
 		
 #define ADD_STR_VALUE( ptr, ptr_size, data) \
-	if (ptr_size <= strlen(data)+strlen(ptr)) { \
+	*len = strlen(data) + 1; \
+	if (ptr_size < strlen(ptr)+(*len)) { \
 		return ASN_MEM_ERROR; \
 	} else { \
-		strcat( ptr, data); \
-		*len = strlen(ptr)+1; \
+		/* this strcat is checked */ \
+		strcat(ptr, data); \
 	}
-		
+
 /**
   * asn1_read_value - Returns the value of one element inside a structure
   * @root: pointer to a structure
   * @name: the name of the element inside a structure that you want to read.
   * @value: vector that will contain the element's content. 
   * VALUE must be a pointer to memory cells already allocated.
-  * @len: number of bytes of *value: value[0]..value[len-1]
+  * @len: number of bytes of *value: value[0]..value[len-1]. Initialy holds the sizeof value.
+  *
   * Description:
   *
   * Returns the value of one element inside a structure.
@@ -1339,7 +1363,7 @@ asn1_write_value(node_asn *node_root,char *name,unsigned char *value,int len)
   * 
   **/
 int 
-asn1_read_value(node_asn *root,char *name,unsigned char *value,int *len)
+asn1_read_value(node_asn *root,char *name,unsigned char *value, int *len)
 {
   node_asn *node,*p;
   int len2,len3;
@@ -1388,7 +1412,7 @@ asn1_read_value(node_asn *root,char *name,unsigned char *value,int *len)
     break;
   case TYPE_OBJECT_ID:
     if(node->type&CONST_ASSIGN){
-      strcpy(value,"");
+      _gnutls_str_cpy(value, *len, "");
       p=node->down;
       while(p){
 	if(type_field(p->type)==TYPE_CONSTANT){
@@ -1481,7 +1505,9 @@ _asn1_check_identifier(node_asn *node)
   p=node;
   while(p){
     if(type_field(p->type)==TYPE_IDENTIFIER){
-      strcpy(name2,node->name);strcat(name2,".");strcat(name2,p->value);
+      _gnutls_str_cpy(name2, sizeof(name2), node->name);
+      _gnutls_str_cat(name2, sizeof(name2), ".");
+      _gnutls_str_cat(name2, sizeof(name2), p->value);
       p2=_asn1_find_node(node,name2);
       if(p2==NULL){printf("%s\n",name2); return ASN_IDENTIFIER_NOT_FOUND;} 
     }
@@ -1490,7 +1516,9 @@ _asn1_check_identifier(node_asn *node)
       p2=p->down;
       if(p2 && (type_field(p2->type)==TYPE_CONSTANT)){
 	if(p2->value && !isdigit(p2->value[0])){
-	  strcpy(name2,node->name);strcat(name2,".");strcat(name2,p2->value);
+	  _gnutls_str_cpy(name2, sizeof(name2), node->name);
+	  _gnutls_str_cat(name2, sizeof(name2), ".");
+	  _gnutls_str_cat(name2, sizeof(name2), p2->value);
 	  p2=_asn1_find_node(node,name2);
 	  if(!p2 || (type_field(p2->type)!=TYPE_OBJECT_ID) ||
 	     !(p2->type&CONST_ASSIGN)) 
@@ -1629,7 +1657,9 @@ _asn1_expand_identifier(node_asn **node,node_asn *root)
   while(!((p==*node) && (move==UP))){
     if(move!=UP){
       if(type_field(p->type)==TYPE_IDENTIFIER){
-	strcpy(name2,root->name);strcat(name2,".");strcat(name2,p->value);
+	_gnutls_str_cpy(name2, sizeof(name2), root->name);
+	_gnutls_str_cat(name2, sizeof(name2), ".");
+	_gnutls_str_cat(name2, sizeof(name2), p->value);
 	p2=_asn1_copy_structure2(root,name2);
 	if(p2==NULL) return ASN_IDENTIFIER_NOT_FOUND;
 	_asn1_set_name(p2,p->name);
@@ -1803,7 +1833,7 @@ _asn1_expand_object_id(node_asn *node)
  
   if(node==NULL) return ASN_ELEMENT_NOT_FOUND;
 
-  strcpy(name_root,node->name);
+  _gnutls_str_cpy(name_root, sizeof(name_root), node->name);
 
   p=node;
   move=DOWN;
@@ -1814,7 +1844,9 @@ _asn1_expand_object_id(node_asn *node)
 	p2=p->down;
         if(p2 && (type_field(p2->type)==TYPE_CONSTANT)){
 	  if(p2->value && !isdigit(p2->value[0])){
-	    strcpy(name2,name_root);strcat(name2,".");strcat(name2,p2->value);
+	    _gnutls_str_cpy(name2, sizeof(name2), name_root);
+	    _gnutls_str_cat(name2, sizeof(name2), ".");
+	    _gnutls_str_cat(name2, sizeof(name2), p2->value);
 	    p3=_asn1_find_node(node,name2);
 	    if(!p3 || (type_field(p3->type)!=TYPE_OBJECT_ID) ||
 	       !(p3->type&CONST_ASSIGN)) return ASN_ELEMENT_NOT_FOUND;
