@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2001 Nikos Mavroyanopoulos
+ *      Copyright (C) 2001,2002 Nikos Mavroyanopoulos
  *
  * This file is part of GNUTLS.
  *
@@ -21,7 +21,7 @@
 #include <gnutls_int.h>
 #include <gnutls_errors.h>
 #include <x509_b64.h>
-#include <auth_x509.h>
+#include <auth_cert.h>
 #include <gnutls_cert.h>
 #include <x509_asn1.h>
 #include <x509_der.h>
@@ -31,7 +31,10 @@
 #include <gnutls_global.h>
 #include <gnutls_pk.h>
 #include <debug.h>
+#include <gnutls_str.h>
 
+/* returns DER tbsCertificate
+ */
 static gnutls_datum _gnutls_get_tbs( gnutls_cert* cert) {
 node_asn *c2;
 gnutls_datum ret = {NULL, 0};
@@ -80,7 +83,7 @@ int result;
 opaque str[1024];
 int len;
 
-	if (asn1_create_structure( _gnutls_get_pkcs(), "PKCS-1.DigestInfo", &dinfo, "digest_info")!=ASN_OK) {
+	if (asn1_create_structure( _gnutls_get_gnutls_asn(), "GNUTLS.DigestInfo", &dinfo, "digest_info")!=ASN_OK) {
 		gnutls_assert();
 		return GNUTLS_E_ASN1_ERROR;
 	}
@@ -111,9 +114,9 @@ int len;
 	}
 
 	if (*hash==-1) {
-#ifdef DEBUG
-		_gnutls_log( "HASH OID: %s\n", str);
-#endif
+
+		_gnutls_x509_log( "X509_SIG: HASH OID: %s\n", str);
+
 		gnutls_assert();
 		return GNUTLS_E_UNIMPLEMENTED_FEATURE;
 	}
@@ -133,11 +136,11 @@ int len;
 
 /* if hash==MD5 then we do RSA-MD5
  * if hash==SHA then we do RSA-SHA
- * m is modulus
- * e is public key
+ * params[0] is modulus
+ * params[1] is public key
  */
-int
-_pkcs1_rsa_verify_sig( gnutls_datum* signature, gnutls_datum* text, MPI e, MPI m)
+static int
+_pkcs1_rsa_verify_sig( const gnutls_datum* signature, gnutls_datum* text, MPI *params)
 {
 	MACAlgorithm hash;
 	int ret;
@@ -146,8 +149,7 @@ _pkcs1_rsa_verify_sig( gnutls_datum* signature, gnutls_datum* text, MPI e, MPI m
 	GNUTLS_HASH_HANDLE hd;
 	gnutls_datum decrypted;
 	
-
-	if ( (ret=_gnutls_pkcs1_rsa_decrypt( &decrypted, *signature, e, m, 1)) < 0) {
+	if ( (ret=_gnutls_pkcs1_rsa_decrypt( &decrypted, *signature, params, 1)) < 0) {
 		gnutls_assert();
 		return ret;
 	}
@@ -156,7 +158,7 @@ _pkcs1_rsa_verify_sig( gnutls_datum* signature, gnutls_datum* text, MPI e, MPI m
 	 */
 
 	digest_size = sizeof(digest);	
-	if ( (ret = _gnutls_get_ber_digest_info( &decrypted, &hash, digest, &digest_size )) != 0) {
+	if ( (ret = _gnutls_get_ber_digest_info( &decrypted, &hash, digest, &digest_size)) != 0) {
 		gnutls_assert();
 		gnutls_sfree_datum( &decrypted);
 		return ret;
@@ -181,130 +183,57 @@ _pkcs1_rsa_verify_sig( gnutls_datum* signature, gnutls_datum* text, MPI e, MPI m
 	return 0;		
 }
 
+/* verifies if the certificate is properly signed.
+ * returns 0 on success.
+ */
 CertificateStatus gnutls_x509_verify_signature(gnutls_cert* cert, gnutls_cert* issuer) {
 gnutls_datum signature;
 gnutls_datum tbs;
 
-	if ( issuer->subject_pk_algorithm == GNUTLS_PK_RSA) {
-		signature.data = cert->signature;
-		signature.size = cert->signature_size;
+	signature.data = cert->signature.data;
+	signature.size = cert->signature.size;
 		
-		tbs = _gnutls_get_tbs( cert);
-		if (tbs.data==NULL) {
-			gnutls_assert();
-			return GNUTLS_CERT_INVALID;
-		}
-		
-		if (_pkcs1_rsa_verify_sig( &signature, &tbs, issuer->params[1], issuer->params[0])!=0) {
-			gnutls_assert();
-			gnutls_free_datum( &tbs);
-			return GNUTLS_CERT_NOT_TRUSTED;
-		}
-		gnutls_free_datum(&tbs);
-		return GNUTLS_CERT_TRUSTED;
+	tbs = _gnutls_get_tbs( cert);
+	if (tbs.data==NULL) {
+		gnutls_assert();
+		return GNUTLS_CERT_INVALID;
 	}
-#ifdef DEBUG
-	_gnutls_log( "PK: %d\n", issuer->subject_pk_algorithm);	
-#endif
+
+	_gnutls_x509_log("X509_VERIFY: CERT[%s]\n", GET_CN(cert->raw));
+	_gnutls_x509_log("X509_VERIFY: ISSUER[%s]\n", GET_CN(issuer->raw));
+
+	switch( issuer->subject_pk_algorithm) {
+		case GNUTLS_PK_RSA:
+		
+			if (_pkcs1_rsa_verify_sig( &signature, &tbs, issuer->params)!=0) {
+				gnutls_assert();
+				gnutls_free_datum( &tbs);
+				return GNUTLS_CERT_INVALID;
+			}
+
+			gnutls_free_datum(&tbs);
+			return 0;
+			break;
+
+		case GNUTLS_PK_DSA:
+			if (_gnutls_dsa_verify( &tbs, &signature, issuer->params)!=0) {
+				gnutls_assert();
+				gnutls_free_datum( &tbs);
+				return GNUTLS_CERT_INVALID;
+			}
+
+			gnutls_free_datum(&tbs);
+			return 0;
+			break;
+
+	}
+
+	gnutls_free_datum(&tbs);
+
+	_gnutls_x509_log( "X509_SIG: PK: %d\n", issuer->subject_pk_algorithm);	
 
 	gnutls_assert();
 	return GNUTLS_CERT_INVALID;
 }
 
 
-#if 0
-/* Signature generation - not tested */
-static int _gnutls_digestinfo_encode( opaque* data, int data_size, char* OID, gnutls_datum* der) {
-node_asn *di;
-int result;
-
-	if (asn1_create_structure( _gnutls_get_pkcs(),
-                    "PKCS-1.DigestInfo", &di, "di") != ASN_OK) {
-        	gnutls_assert();
-		return GNUTLS_E_ASN1_ERROR;
-	}
-	
-	result = asn1_write_value( di, "di.digestAlgorithm.algorithm", OID, 1);
-	if (result!=ASN_OK) {
-        	gnutls_assert();
-		asn1_delete_structure( di);
-		return GNUTLS_E_ASN1_ERROR;
-	}
-
-	result = asn1_write_value( di, "di.digestAlgorithm.parameters", NULL, 0);
-	if (result!=ASN_OK) {
-        	gnutls_assert();
-		asn1_delete_structure( di);
-		return GNUTLS_E_ASN1_ERROR;
-	}
-
-	result = asn1_write_value( di, "di.digest", data, data_size);
-	if (result!=ASN_OK) {
-        	gnutls_assert();
-		asn1_delete_structure( di);
-		return GNUTLS_E_ASN1_ERROR;
-	}
-
-	der->size = data_size + 200;
-	der->data = gnutls_malloc( der->size);
-	if (der->data==NULL) {
-		gnutls_assert();
-		asn1_delete_structure( di);
-		return GNUTLS_E_MEMORY_ERROR;
-	}
-	
-	result = asn1_create_der( di, "di", der->data, &der->size);
-	if (result!=ASN_OK) {
-        	gnutls_assert();
-		asn1_delete_structure( di);
-        	gnutls_free_datum( der);
-		return GNUTLS_E_ASN1_ERROR;
-	}
-	asn1_delete_structure( di);
-
-	return 0;
-}
-
-int _pkcs1_rsa_generate_sig( MACAlgorithm hash_algo, gnutls_private_key *pkey, const gnutls_datum *data, gnutls_datum *signature) {
-	int ret;
-	GNUTLS_HASH_HANDLE hd;
-	opaque digest[MAX_HASH_SIZE];
-	char OID[40];
-	int digest_size =  gnutls_hash_get_algo_len( hash_algo);
-	gnutls_datum der;
-	
-	if (hash_algo==GNUTLS_MAC_MD5)
-		strcpy(OID, "1 2 840 113549 2 5");
-	else if (hash_algo==GNUTLS_MAC_SHA)
-		strcpy(OID, "1 3 14 3 2 26");
-	else {
-		gnutls_assert();
-		return GNUTLS_E_UNKNOWN_MAC_ALGORITHM;
-	}
-	
-	/* hash data */
-	hd = gnutls_hash_init( hash_algo);
-	if (hd==NULL) {
-		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
-	}
-	gnutls_hash( hd, data->data, data->size);
-	gnutls_hash_deinit( hd, digest);
-
-	/* encode digest to DigestInfo (der) */
-	if ( (ret=_gnutls_digestinfo_encode( digest, digest_size, OID, &der)) < 0) {
-		gnutls_assert();
-		return ret;
-	}	
-
-	der.data = digest;
-	der.size = digest_size;
-	/* encrypt der */
-	if ( (ret=_gnutls_pkcs1_rsa_encrypt( signature, der, pkey->params[0], pkey->params[1], 1)) < 0) {
-	     gnutls_assert();
-	     return ret;
-	}
-
-	return 0;
-}
-#endif
