@@ -1,7 +1,7 @@
 #define PORT 5556
 #define SERVER "127.0.0.1"
 
-#include <gnutls.h>
+#include "../lib/gnutls.h"
 
 int print_info( GNUTLS_STATE state);
 int print_list(void);
\ No newline at end of file
